
#!python3.x
# Sample demo to use FC612 and Trigger-Extension FC1015

import sys, os
import platform
import time, datetime
import json
import operator

from tkinter import *

# the back-end logic to control the Adapter
from app_raw_loop import AppRawLoop
from select_adapter import SelectAdapter

FontNormal = ('Arial', 10)
FontLarge = ('Arial', 10)
FontHuge = ('Arial', 18)

AppVersion = '1.0.0'

class AppGui:
    
    def __init__(self, width=0, height=0):

        self.root = Tk()

        self.app = AppRawLoop()
        #Basic Window Geometry

        self.root.geometry
        self.root.grid(widthInc=width,heightInc=height)


        #Root-Window Apperace

        self.root.configure(background='#44474c')
        self.root.title('Loopback Trigger IO Demo Ext. V ' + AppVersion)
        self.root.resizable(width=FALSE, height=FALSE)

        # gui variables
        self.lstRxCnt = 0
        self.lstTxCnt = 0

        self.TxCounter = 0

        #Call Function to Create the Slave Widgets

        self.createWindow()

        #Event Bind

        self.root.bind('<Destroy>',self.shutdown)

        #Mainloop
        self.update_clock()
        self.root.mainloop()


    def update_clock(self):
        #now = time.strftime("%H:%M:%S")
        self.update_gui()
        self.root.after(200, self.update_clock)

    def createWindow(self):
        """
        ----------------------------------------------------------------
         createWindow(self)

          Function: Build the Slave Widgets of the Root  - Window
        -----------------------------------------------------------------
        """

        self.topFrame = Frame(self.root)
        self.topFrame.grid(column=0,row=0,sticky=W)

        self.mainFrame = Frame(self.root)
        self.mainFrame.grid(column=0,row=1,sticky=W)

        self.statusFrame = Frame(self.root)
        self.statusFrame.grid(column=0,row=2,sticky=W)

        self.btnSelect = Button(self.topFrame, font=FontLarge, width=18, text='Select ...', command=self.on_btn_select_click)
        self.btnSelect.grid(column=0, row=0, sticky=W)

        lbOpen = Label(self.topFrame, font = FontLarge,  width=8, text='Open')
        lbOpen.grid(column=1,row=0,sticky=W)
        self.adapterOpened = BooleanVar()
        
        self.cbOpen = Checkbutton(self.topFrame, font = FontLarge, variable=self.adapterOpened, command=self.cb_open_changed)
        self.cbOpen.config(state = DISABLED)
        self.cbOpen.grid(column=2,row=0,sticky=W)

        lbLoopback = Label(self.topFrame, font = FontLarge, width=8, text='Loopback')
        lbLoopback.grid(column=3,row=0,sticky=W)
        
        self.loopBackEnabled = BooleanVar()

        self.cbLoopback = Checkbutton(self.topFrame, font = FontLarge, variable=self.loopBackEnabled, command=self.cb_loopback_changed)
        self.cbLoopback.config(state = DISABLED)
        self.cbLoopback.grid(column=4,row=0,sticky=W)

        
        self.btnTrigOut = Button(self.topFrame, font=FontLarge, text='Trigger-Out', command=self.btn_click_trig_out)
        self.btnTrigOut.grid(column=5, row=0, sticky=W)

        self.btnSend = Button(self.topFrame, font=FontLarge, text='Send Raw-Tx-Frame', command=self.btn_click_raw_send)
        self.btnSend.config(state=DISABLED)
        self.btnSend.grid(column=6, row=0, sticky=W)

        self.btnClear = Button(self.topFrame, font=FontLarge, width=17, text='Clear', command=self.btn_click_clear)
        self.btnClear.grid(column=7, row=0, sticky=W)


         # Main-Frame list of Tx Frames
        self.lbTxFrames = Listbox(self.mainFrame, font = FontLarge, selectmode='browse')
        self.lbTxFrames.grid(column=0,row=0,sticky=W)
        self.lbTxFrames.config(width=100)

        # Main-Frame list of Rx Frames
        self.lbRxFrames = Listbox(self.mainFrame, font = FontLarge, selectmode='browse')
        self.lbRxFrames.grid(column=0,row=1,sticky=W)
        self.lbRxFrames.config(width=100)

        # Status-Frame
        self.selectedAdapterName = StringVar()

        lbSelectedAdapter = Label(self.statusFrame, font = FontLarge, width=15, anchor="w", textvariable=self.selectedAdapterName)
        lbSelectedAdapter.grid(column=0,row=0,sticky=W)

        lbIdle = Label(self.statusFrame, font = FontLarge, anchor="w", text='  IDLE: ')
        lbIdle.grid(column=2,row=0,sticky=W)

        self.lastIdleFrame = StringVar()

        lbIdleFrame = Label(self.statusFrame, font = FontLarge, width=24, anchor="w", textvariable=self.lastIdleFrame)
        lbIdleFrame.grid(column=3,row=0,sticky=W)

        lbTxRxTime = Label(self.statusFrame, font = FontLarge, anchor="w", text = 'TX-RX Diff: ')
        lbTxRxTime.grid(column=4,row=0,sticky=W)

        self.lastTxRxDiffTime = StringVar()

        lbTxRxDiffTime = Label(self.statusFrame, font = FontLarge, width=10, anchor="w", textvariable=self.lastTxRxDiffTime)
        lbTxRxDiffTime.grid(column=5,row=0,sticky=W)

        lbRxTrig = Label(self.statusFrame, font = FontLarge, anchor="w", text = 'RX-TRIG Diff: ')
        lbRxTrig.grid(column=6,row=0,sticky=W)

        self.lastRxTrigDiffTime = StringVar()

        lbRxTrigDiffTime = Label(self.statusFrame, font = FontLarge, width=10, anchor="w", textvariable=self.lastRxTrigDiffTime)
        lbRxTrigDiffTime.grid(column=7,row=0,sticky=W)
        

    def update_gui(self):
        #self.stickTime.set(self.app.get_stick_time())
        rawRxList = self.app.get_rx_frames()
        cnt = self.lstRxCnt
        for entry in rawRxList:
            if entry.FrameType == 2:
                rxFrame = '{0}.{1}'.format(entry.TimestampSec, entry.TimestampNano)
                self.lastIdleFrame.set('{0:04} Idle-Frame: {1}'.format(cnt, rxFrame))
                cnt += 1
            elif entry.FrameType == 1:
                rxFrame = '{0}.{1}'.format(entry.TimestampSec, entry.TimestampNano)
                self.lbRxFrames.insert('end', '{0:4} RxFrame: {1}'.format(cnt, rxFrame))
                cnt += 1
            elif entry.FrameType == 3:
                triggerFrame = '{0}.{1}'.format(entry.TimestampSec, entry.TimestampNano)
                self.lbRxFrames.insert('end', '{0:4} Trigger: {1}'.format(cnt, triggerFrame))
                cnt += 1
        self.lbRxFrames.see(END)
        self.lstRxCnt = cnt

        rawTxList = self.app.get_tx_frames()
        cnt = self.lstTxCnt
        for entry in rawTxList:
            cnt += 1
            txFrameData = '{0}.{1} {2}'.format(entry[1].seconds, entry[1].nanosec, entry[2][0:32])
            self.lbTxFrames.insert('end', '{0:4} {1}'.format(cnt, txFrameData))
        self.lbTxFrames.see(END)
        self.lstTxCnt = cnt

        self.lastTxRxDiffTime.set('{0:8} ns'.format(self.app.get_last_rx_diff_time().nanosec))
        self.lastRxTrigDiffTime.set('{0:8} ns'.format(self.app.get_last_trig_diff_time().nanosec))

    # EVENT Handler
    def shutdown(self, event):
        """close all open instances to clean-up"""
        self.app.close()

    def on_btn_select_click(self):
        self.select = SelectAdapter(self.root, title='Select FibreCode USB Stick Raw', adapterDic=self.app.get_adapter_dic()).show()
        if self.select == None:
            self.cbOpen.config(state = DISABLED)
            return
        self.selectedAdapterName.set('{0:2}: {1} / {2}'.format(self.select['Inst'], self.select['DeviceName'], self.select['Name']))
        self.cbOpen.config(state = ACTIVE)
        

    def cb_open_changed(self):
        state = self.adapterOpened.get()
        if state:
            inst = self.select['Inst']
            self.app.open(inst)
            self.app.control_trigger_in(True)
            self.app.control_idle(500)
            self.app.config_trigger_out([0x88F7], 50)
            self.btnSend.config(state=ACTIVE)
            self.cbLoopback.config(state = ACTIVE)
        else:
            self.app.close()
            self.btnSend.config(state=DISABLED)

    def cb_loopback_changed(self):
        state = self.loopBackEnabled.get()
        self.app.config_phy_loopback(state)

    def btn_click_clear(self):
        self.app.reset()
        self.lbRxFrames.delete(0,END)
        self.lbTxFrames.delete(0,END)

    def btn_click_trig_out(self):
        self.app.force_trigger_out()

    def btn_click_raw_send(self):
        payload = 128
        payloadData = ''

        for j in range(0, payload):
            payloadData += '{0:02X}'.format(j & 0xFF)
        sendData = 'FFFFFFFFFFFF' + self.app.get_mac_address() + '88F7' + '{0:04X}'.format(self.TxCounter) + payloadData

        self.TxCounter += 1

        self.app.send_tx_frame(sendData)


def main(args):
    
    # will never return until gui exits
    app = AppGui(350, 100)

if __name__ == "__main__":
    main(sys.argv)

